/* -----------------------------------------------------------------------------
 * Copyright (c) 2017 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        10. April 2017
 * $Revision:    V1.0.0
 *
 * Project:      Flash Programming Functions for Kinetis K32W
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.0.0
 *    Initial release
 */

#include <stdint.h>
#include "..\FlashOS.h"        /* FlashOS Structures */
#include "fsl_flash.h"


#if   defined P1024_4KB_SEC || defined P256_2KB_SEC
  #define SECTOR_SIZE       FSL_FEATURE_FLASH_PFLASH_BLOCK_SECTOR_SIZE
  #define WRITE_UNIT_SIZE   FSL_FEATURE_FLASH_PFLASH_BLOCK_WRITE_UNIT_SIZE
#else
  #error "SECTOR_SIZE not configured."
#endif


flash_config_t      flash_driver;

/* This function initializes all the RO, RW data in the PI library.
   It is called from the C library startup code.
   The flash algorithm does not execute the C library startup code
   so we must call it manually.
 */
//extern void __cpp_initialize__aeabi_(void);

/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */
int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {
  status_t ret;

//  __cpp_initialize__aeabi_();

  /* set run mode (remove low power mode) */
  RunMode_Set();

  /* disable the watchdog */
  Watchdog_Disable();

#if   defined (P1024_4KB_SEC)
  flash_driver.FlashMemoryIndex = (uint8_t)kFLASH_MemoryIndexPrimaryFlash;
#elif defined (P256_2KB_SEC)
  flash_driver.FlashMemoryIndex = (uint8_t)kFLASH_MemoryIndexSecondaryFlash;
#else
  #error "FlashMemoryIndex not configured."
#endif

  /* initialize global flash properties structure */
  ret = FLASH_Init(&flash_driver);
  if (ret == kStatus_FLASH_Success)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}


/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {

  return (0);                                                 /* Finished without Errors */
}


/*  Blank Check Block in Flash Memory
 *    Parameter:      adr:  Block Start Address
 *                    sz:   Block Size (in bytes)
 *                    pat:  Block Pattern
 *    Return Value:   0 - OK,  1 - Failed
 */
#if 0
int BlankCheck (unsigned long adr, unsigned long sz, unsigned char pat) {
  int ret;

  ret = FLASH_VerifyErase(&flash_driver, adr, sz, kFLASH_MarginValueNormal);
  if (ret == kStatus_FLASH_Success)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}
#endif

/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */
int EraseChip (void) {
  status_t ret;
	uint32_t fsec[2];
//	uint32_t fopt[2];

//  ret = FLASH_ReadOnce(&flash_driver, 0x80, &fsec[0], 2);
//  ret = FLASH_ReadOnce(&flash_driver, 0xA0, &fopt[0], 4);

  ret = FLASH_EraseAll(&flash_driver, kFLASH_ApiEraseKey);
  if (ret != kStatus_FLASH_Success)
  {
    return (1);
  }
	
  ret = FLASH_VerifyEraseAll(&flash_driver, kFLASH_MarginValueNormal);
  if (ret != kStatus_FLASH_Success)
  {
    return (1);
  }

  /* program FSEC FOPT bytes */
	fsec[0] = 0xFFFFFF00;
	fsec[1] = 0xFFFFFFFF;
  ret = FLASH_ProgramOnce(&flash_driver, 0x80, &fsec[0], 2);
//  if (ret != kStatus_FLASH_Success)
//  {
//    return (1);
//  }

//	fopt[0] = 0xFFFFF3FF;
//	fopt[1] = 0xFFFFFFFF;
//  ret = FLASH_ProgramOnce(&flash_driver, 0x84, &fopt[0], 4);
  if (ret == kStatus_FLASH_Success)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */
int EraseSector (unsigned long adr) {
  int ret;

  ret = FLASH_Erase(&flash_driver, adr, SECTOR_SIZE, kFLASH_ApiEraseKey);
  if (ret == kStatus_FLASH_Success)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */
int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {
  status_t ret;

  sz = (sz + (WRITE_UNIT_SIZE - 1)) & ~(WRITE_UNIT_SIZE - 1);     /* Align to WRITE_UNIT_SIZE */

  ret = FLASH_Program(&flash_driver, adr, (uint32_t *)buf, sz);
  if (ret == kStatus_FLASH_Success)
  {
    return (0);                                               /* Finished without Errors */
  }
  else
  {
    return (1);                                               /* Finished with Errors */
  }
}
